from boto.dynamodb.types import Dynamizer


class ResultSet(object):
    """Result of the scan & query operation of the model."""

    def __init__(self, model, operation, kwargs):
        self.model = model
        self.operation = operation
        self.kwargs = kwargs

    def __iter__(self):
        """Result items of the operation."""
        operation = self._get_operation()
        kwargs = self.kwargs.copy()
        while True:
            result = operation(self.model.get_table_name(), **kwargs)
            for raw_item in result.get('Items'):
                yield self.model.from_raw_data(raw_item)

            last_evaluated_key = self._get_last_key(result)
            if last_evaluated_key:
                kwargs['exclusive_start_key'] = last_evaluated_key
            else:
                raise StopIteration

    def count(self):
        """Total count of the matching items.

        It sums up the count of partial results, and returns the total count of
        matching items in the table.
        """
        count = 0
        operation = self._get_operation()
        kwargs = self.kwargs.copy()
        kwargs['select'] = 'COUNT'
        while True:
            result = operation(self.model.get_table_name(), **kwargs)
            count += result['Count']

            last_evaluated_key = self._get_last_key(result)
            if last_evaluated_key:
                kwargs['exclusive_start_key'] = last_evaluated_key
            else:
                break
        return count

    def _get_operation(self):
        return getattr(self.model._get_connection(), self.operation)

    def _get_last_key(self, result):
        last_evaluated_key = result.get('LastEvaluatedKey')
        if not last_evaluated_key:
            return None
        last_key = {}
        dynamizer = Dynamizer()
        for key, value in last_evaluated_key.items():
            last_key[key] = dynamizer.decode(value)
        return last_key



from boto.dynamodb2.layer1 import DynamoDBConnection

from .model import Model


def patch_from_config(config):
    if 'DYNAMODB_CONNECTION' in config:
        patch_dynamodb_connection(**config['DYNAMODB_CONNECTION'])
    if 'DYNAMODB_PREFIX' in config:
        patch_table_name_prefix(config['DYNAMODB_PREFIX'])


def patch_dynamodb_connection(**kwargs):
    """:class:`boto.dynamodb2.layer1.DynamoDBConnection` patcher.

    It partially applies the keyword arguments to the
    :class:`boto.dynamodb2.layer1.DynamoDBConnection` initializer method.
    The common usage of this function would be patching host and port
    to the local DynamoDB or remote DynamoDB as the project configuration
    changes.

    """
    if hasattr(DynamoDBConnection, '__original_init__'):
        return

    DynamoDBConnection.__original_init__ = DynamoDBConnection.__init__

    def init(self, **fkwargs):
        fkwargs.update(kwargs)
        self.__original_init__(**fkwargs)

    DynamoDBConnection.__init__ = init


def patch_table_name_prefix(prefix):
    """Patch the table name prefix"""

    Model._table_prefix = prefix



from setuptools import find_packages, setup

from bynamodb.version import VERSION


install_requires = ['boto']

setup(
    name='bynamodb',
    version=VERSION,
    url='https://github.com/teddychoi/BynamoDB',
    author='Bochul Choi',
    author_email='vio.bo94@gmail.com',
    license='MIT',
    description='High-Level DynamoDB Interface for Python'
                'wrapping Low-Level Interface of boto',
    install_requires=install_requires,
    packages=find_packages(exclude=['tests'])
)



class Index(object):
    """Declare the index of the model as a class."""

    # (:class:`str`) The attribute name used for the index hash key.
    hash_key = None

    # (:class:`str`) The attribute name used for the index hash key.
    range_key = None

    #: (:class:`str`) The index name.
    #: # If omitted, the Index class name will be the table name.
    index_name = None

    #: (:class:`str`) The projection type of the index.
    projection_type = None

    _keys = None

    @classmethod
    def schema(cls):
        return {
            'IndexName': cls._get_index_name(),
            'KeySchema': [key.schema() for key in cls._keys],
            'Projection': {
                'ProjectionType': cls.projection_type
            }
        }

    @classmethod
    def _get_index_name(cls):
        return cls.index_name or cls.__name__


class GlobalIndex(Index):
    read_throughput = None
    write_throughput = None

    @classmethod
    def schema(cls):
        schema = super(GlobalIndex, cls).schema()
        schema['ProvisionedThroughput'] = {
            'ReadCapacityUnits': int(cls.read_throughput),
            'WriteCapacityUnits': int(cls.write_throughput)
        }
        return schema


class AllIndex(Index):
    projection_type = 'ALL'


class GlobalAllIndex(GlobalIndex):
    projection_type = 'ALL'
